@@ -11,6 +11,7 @@
 
 #include "scriptwriter.h"
 #include "firestarter.h"
+#include "portfw.h"
 #include <libgnomeui/gnome-window-icon.h>
 
 static GdkImlibImage *image;
@@ -301,6 +302,8 @@ create_masq_page (Druid *data)
 	GtkWidget *radio3;
 	GtkWidget *entry;
 	GtkWidget *combo;
+	GtkWidget *align;
+	GtkWidget *button;
 	GdkImlibImage *logo;
 
 	logo = gdk_imlib_load_image("/usr/share/pixmaps/firestarter/masq.png");
@@ -414,11 +417,26 @@ create_masq_page (Druid *data)
 	data->intdevice = (GTK_COMBO (combo)->entry);
 	data->intrange = entry;
 
+
+	align = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
+//	gtk_table_attach (GTK_TABLE (table2), align, 0, 1, 4, 5,
+//		GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+
+	button = gtk_button_new_with_label (_("Port Forwarding"));
+	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			    GTK_SIGNAL_FUNC (show_portfw_window), NULL);
+	gtk_container_add (GTK_CONTAINER (align), button);
+
+
 	gtk_widget_show_all (table);
 
 	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (page)->vbox),
 			    table, TRUE, TRUE, 0);
 
+	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (page)->vbox),
+			    align, TRUE, TRUE, 5);
+
+
 	gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (page), &white);
 	gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (page), &white);
 
@@ -442,7 +460,7 @@ create_services_page (Druid *data)
 
 	turn_white (page);
 
-	table = gtk_table_new (4, 1, FALSE);
+	table = gtk_table_new (5, 1, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
 	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
 
@@ -544,7 +562,15 @@ create_services_page (Druid *data)
 	gtk_box_pack_start (GTK_BOX (vbox), data->dhcp, FALSE, FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (data->services), vbox, FALSE, FALSE, 0);
+	
+	/* Column 4 */
+	vbox = gtk_vbox_new (FALSE, 1);
 
+	data->ipsec = gtk_check_button_new_with_label ("IPSEC");
+	gtk_box_pack_start (GTK_BOX (vbox), data->ipsec, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (data->services), vbox, FALSE, FALSE, 0);
+	
 	gtk_widget_show_all (table);
 
 	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (page)->vbox),
@@ -720,7 +746,7 @@ create_tos_page (Druid *data)
 
 	gtk_tooltips_set_tip (ttips, data->tos_server, _(
 		"Check this option to enable Type of Service flags for common "
-		"server applications, such as SQUID, FTPd, SSHd, STMP & POP Daemons."), "");
+		"server applications, such as SQUID, FTPd, SSHd, SMTP & POP Daemons."), "");
 
 	data->tos_X = gtk_check_button_new_with_label ("The X Window System");
 	gtk_box_pack_start (GTK_BOX (vbox), data->tos_X, FALSE, FALSE, 0);
