@@ -30,7 +30,7 @@ struct _Druid
 	GtkWidget *tos;
 	GtkWidget *masq;
 	GtkWidget *ftp, *ssh, *telnet, *smtp, *dns, *finger, *www, *sslweb,
-		  *pop, *imap, *ident, *nntp, *ntp, *samba, *routed, *nfs,
+		  *pop, *imap, *ident, *nntp, *ntp, *samba, *ipsec, *routed, *nfs,
 		  *x, *dhcp;
 
 	GtkWidget *icmp_echo, *icmp_traceroute, *icmp_tracert, *icmp_unreach,
