@@ -1,5 +1,5 @@
- /*---[ firestarter.c ]-----------------------------------------------
- * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
+/*---[ firestarter.c ]------------------------------------------------
+ * Copyright (C) 2000, 2001 Tomas Junnonen (majix@sci.fi)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,163 +11,70 @@
 
 #include <config.h>
 #include <gnome.h>
-
 #include <status-docklet.h>
-
-#include <errno.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <unistd.h>
+#include <stdio.h>
+#include <errno.h>
 
 #include "firestarter.h"
+#include "util.h"
+#include "logread.h"
 #include "menus.h"
 #include "parse.h"
 #include "druid.h"
 #include "preferences.h"
 #include "modrules.h"
+#include "sort-clist.h"
 
 #include "xpm/statusrunning.xpm"
-#include "xpm/statuscaught.xpm"
+/*#include "xpm/statuscaught.xpm"*/
 #include "xpm/statustopped.xpm"
 
-#include <libgnomeui/gnome-window-icon.h>
-
 gboolean NETFILTER;
-gboolean running = FALSE;
 
 GtkWidget *window;
 GtkWidget *panelvbox;
 GtkWidget *panelpic;
 
+static gint save_session (GnomeClient       *client,
+			  gint               phase,
+			  GnomeSaveStyle     save_style,
+			  gint               is_shutdown,
+			  GnomeInteractStyle interact_style,
+			  gint               is_fast,
+			  gpointer           client_data);
+
+static void session_die  (GnomeClient        *client,
+			  gpointer            client_data);
+
 void set_panel_pic (gchar *pixmap[]);
 
-/* [ check_version ]
- * Check version on the script file. If it was created for the wrong
- * kernel type or the version was < program version, start fresh.
- */
-static void
-check_version (void)
-{
-	FILE *f;
-	gchar *version = "0.0.0";
-	gchar buf[512];
-	gint wrong_kernel = FALSE;
-
-	f = fopen (FIRESTARTER_RULES_DIR "/firestarter/firewall.sh", "r");
-
-	if (!f) {
-		gnome_config_set_bool ("/firestarter/Druid/firsttime", TRUE);
-		gnome_config_sync ();
-		return;
-	}
-
-	while (fgets (buf, 512, f) != NULL) {
-		if (strstr (buf, "by Firestarter "))  {
-			version = get_text_between (buf, "by Firestarter ", ",");
-			if (strstr (buf, "NETFILTER")) {
-				if (!NETFILTER)
-					wrong_kernel = TRUE;
-			} else
-				if (NETFILTER)
-					wrong_kernel = TRUE;
-			break;
-		}
-	}
-
-	fclose (f);
-
-	if ((g_strcasecmp (version, VERSION) < 0) || wrong_kernel) {
-		GtkWidget *dialog;
-		GtkWidget *label;
-		int button;
-
-		mkdir (FIRESTARTER_RULES_DIR "/firestarter", 00700);
-
-		dialog = gnome_dialog_new (_("Old version detected"),
-			GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
-
-		label = gtk_label_new (_(
-			"\nA firewall created by an older version of\n"
-			"Firestarter has been detected.\n\n"
-			"OK to remove old firewall (recommended)?\n"));
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
-			label, FALSE, FALSE, 0);
-		gtk_widget_show (label);
-
-		button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-
-		switch (button) {
-			case 0:
-				remove (FIRESTARTER_RULES_DIR "/firestarter/firewall.sh");
-				remove (FIRESTARTER_RULES_DIR "/firestarter/allow-all");
-				remove (FIRESTARTER_RULES_DIR "/firestarter/deny-all");
-				remove (FIRESTARTER_RULES_DIR "/firestarter/allow-service-all");
-				remove (FIRESTARTER_RULES_DIR "/firestarter/allow-service-machine");
-				remove (FIRESTARTER_RULES_DIR "/firestarter/do-not-log-ports");
-				gnome_config_set_bool ("/firestarter/Druid/firsttime", TRUE);
-				gnome_config_sync ();
-				printf ("Scripts removed");
-				break;
-
-			case 1:
-			case -1:
-				printf ("Scripts not removed");
-				break;
-		}
-	}
-}
-
 /* [ appbar_update ]
- * Updates the application status bar with the state of the firewall
+ * Updates the application status bar with the state of the firewall,
+ * returns the new state
  */
-static void
+static int
 appbar_update (char *state)
 {
-	if (state == "stop")
+	static int currentstate;
+
+	if (state == "stop") {
 		gnome_appbar_set_default (GNOME_APPBAR (appbar),
-					_("Firewall stopped"));
-	if (state == "start")
-		gnome_appbar_set_default (GNOME_APPBAR (appbar),
-					 _("Firewall running"));
-	if (state == "halt")
-		gnome_appbar_set_default (GNOME_APPBAR (appbar),
-					_("All traffic halted"));
-	if (state == "unknown") {
-		if (running)
-			gnome_appbar_set_default (GNOME_APPBAR (appbar),
-						 _("Firewall running"));
-		else
-			gnome_appbar_set_default (GNOME_APPBAR (appbar),
-						_("Firewall stopped"));
+			_("Firewall stopped"));
+		currentstate = 1;
 	}
-}
-
-static void
-detect_netfilter (void)
-{
-	FILE *f;
-	gchar buf[512];
-
-	f = fopen ("/proc/version", "r");
-	if (!f) {
-		g_print ("Could not determine kernel version,"
-			 "defaulting to 2.2/ipchains");
-		return;
+	else if (state == "start") {
+		gnome_appbar_set_default (GNOME_APPBAR (appbar),
+			_("Firewall running"));
+		currentstate = 2;
+	}
+	else if (state == "halt") {
+		gnome_appbar_set_default (GNOME_APPBAR (appbar),
+			_("All traffic halted"));
+		currentstate = 3;
 	}
 
-	while (fgets (buf, 512, f) != NULL) {
-		if (strstr (buf, " 2.4.") != NULL) {
-			g_print ("NETFILTER detected\n");
-			system ("/sbin/insmod ip_tables");
-			NETFILTER = TRUE;
-			fclose (f);
-			return;
-		}
-	}
-	g_print ("IPCHAINS detected\n");
-	NETFILTER = FALSE;
-	fclose (f);
+	return currentstate;
 }
 
 /* [ clist_row_select ]
@@ -225,7 +132,7 @@ hide_window_cb (GtkWidget *window, GdkEventButton* bevent)
 			else
 				gtk_widget_show_all (window);
 
-		if (running)
+		if (appbar_update ("query") == 2)
 			set_panel_pic (statusrunning_xpm);
 	}
 }
@@ -281,7 +188,6 @@ stop_firewall (void)
 	g_free (command);
 
 	g_print (_("Firewall stopped, network traffic is now flowing freely\n"));
-	running = FALSE;
 	set_panel_pic (statustopped_xpm);
 	appbar_update ("stop");
 
@@ -313,7 +219,6 @@ restart_firewall (void)
 
 	system (commandline);
 	g_print (_("Firewall script restarted\n"));
-	running = TRUE;
 
 	set_panel_pic (statusrunning_xpm);
 	g_free (commandline);
@@ -377,23 +282,10 @@ halt_traffic (void)
 	g_free (command);
 
 	g_print (_("All traffic halted\n"));
-	running = FALSE;
 	set_panel_pic (statustopped_xpm);
 	appbar_update ("halt");
 }
 
-/* [ clear_hitlist ]
- * Clears the log CList
- */
-void
-clear_hitlist (void)
-{
-	if (running)
-		set_panel_pic (statusrunning_xpm);
-	gtk_clist_clear (GTK_CLIST (hitlogclist));
-	clist_row_unselect (hitlogclist);
-}
-
 static void
 tail_and_resize (GtkWidget *clist)
 {
@@ -412,6 +304,20 @@ tail_and_resize (GtkWidget *clist)
 	gtk_widget_queue_resize	(window);
 }
 
+
+
+/* [ clear_hitlist ]
+ * Clears the log CList
+ */
+void
+clear_hitlist (void)
+{
+	if (appbar_update ("get current") == 2)
+		set_panel_pic (statusrunning_xpm);
+	gtk_clist_clear (GTK_CLIST (hitlogclist));
+	clist_row_unselect (hitlogclist);
+}
+
 /* [ reload_hitlist ]
  * Reads the entire log file into the hitlogclist
  */
@@ -425,8 +331,8 @@ reload_hitlist (gpointer data)
 	FILE *f;
 
 	f = fopen (logpath, "r");
-	if (!f) {
-		g_print ("Error opening log file");
+	if (f == NULL) {
+		perror(logpath);
 		return;
 	}
 
@@ -446,43 +352,6 @@ reload_hitlist (gpointer data)
 	fclose (f);
 }
 
-/* [ poll_log_timeout ]
- * Polls the logfile every 700ms for change, if change, parse lines
- */
-static int
-poll_log_timeout (gpointer file)
-{
-	gchar buf[512];
-	static gchar *key, *key2;
-	gboolean sound = gnome_config_get_bool (
-		"/firestarter/Sounds/enablesounds=TRUE");
-	gchar *soundfile = gnome_config_get_string (
-		"/firestarter/Sounds/hitsoundfile=");
-
-	if (!key) {
-		if (NETFILTER)
-			key = g_strdup ("kernel: IN=");
-		else {
-			key = g_strdup ("DENY");
-			if (!key2)
-				key2 = g_strdup ("REJECT");
-		}		
-	}
-
-	if (fgets (buf, 512, file) != NULL) {
-		if ( (strstr (buf, key) || (!NETFILTER && strstr (buf, key2))) && parse_hit_log (buf)) {
-			set_panel_pic (statuscaught_xpm);
-			tail_and_resize (hitlogclist);
-
-			if (sound) {
-				gnome_sound_play (soundfile);
-				g_free (soundfile);
-			}
-		}
-	}
-	return TRUE;
-}
-
 /* [ show_about ]
  * Creates the about dialog
  */
@@ -506,7 +375,7 @@ show_about (GtkWidget *widget, gpointer data)
 			"(C) 2000 Tomas Junnonen", authors,
 			_("An all-in-one Linux firewall utility for GNOME.\n"
 			"http://firestarter.sourceforge.net"),
-			"pengo.png");
+			NULL);
 
 		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 			GTK_SIGNAL_FUNC (gtk_widget_destroyed), &dialog);
@@ -547,14 +416,10 @@ status_docklet_build_plug (StatusDocklet *docklet, GtkWidget *plug)
 	panelvbox = gtk_vbox_new (FALSE, 5);
 	gtk_container_add (GTK_CONTAINER (eventbox), panelvbox);
 
-	if (gnome_config_get_bool ("/firestarter/Program/shutdown=FALSE")) {
+	if (gnome_config_get_bool ("/firestarter/Program/shutdown=FALSE"))
 		panelpic = gnome_pixmap_new_from_xpm_d (statustopped_xpm);
-		running = FALSE;
-	}
-	else {
+	else
 		panelpic = gnome_pixmap_new_from_xpm_d (statusrunning_xpm);
-		running = TRUE;
-	}
 
 	gtk_box_pack_start (GTK_BOX (panelvbox), panelpic, FALSE, FALSE, 2);
 
@@ -630,6 +495,7 @@ closewindow (void)
 int
 main (int argc, char* argv[])
 {
+	GnomeClient *client;
 	GtkObject *status_dock;
 
 	GtkWidget *notebook;
@@ -644,9 +510,7 @@ main (int argc, char* argv[])
 	gchar *allowsmtitles[1];
 	gchar *allowsatitles[1];
 
-	gchar *logpath;
 	gint i;
-	FILE *file;
 	CORBA_Environment ev;
 	CORBA_ORB orb;
 
@@ -667,10 +531,17 @@ main (int argc, char* argv[])
 	orb = gnome_CORBA_init (PACKAGE, VERSION, &argc, argv, 0, &ev);
 
 	window = gnome_app_new (PACKAGE, "Firestarter");
-	detect_netfilter ();
+	NETFILTER = detect_netfilter ();
 
 	ttips = gtk_tooltips_new ();
 
+/* Set up the session managment */
+	client = gnome_master_client ();
+	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
+			    GTK_SIGNAL_FUNC (save_session), argv[0]);
+	gtk_signal_connect (GTK_OBJECT (client), "die",
+			    GTK_SIGNAL_FUNC (session_die), NULL);
+
 /* Set up the docklet */
 	status_dock = status_docklet_new ();
 	gtk_signal_connect (GTK_OBJECT (status_dock), "build_plug",
@@ -681,86 +552,68 @@ main (int argc, char* argv[])
 	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			    GTK_SIGNAL_FUNC (closewindow), NULL);
 
-	gnome_window_icon_set_from_file (GTK_WINDOW (window),
-					 "/usr/share/pixmaps/firestarter.png");
+	#if GNOME_LIBS_VERSION>=60
+	gnome_window_icon_set_default_from_file (
+		"/usr/share/pixmaps/firestarter.png");
+	#endif
 
 	install_menus_and_toolbar (window);
 
 	notebook = gtk_notebook_new ();
 	gnome_app_set_contents (GNOME_APP (window), notebook);
 
-/* set up hitlogclist */
+/* Set up hitlogclist */
 	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
-					GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+		GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 
 	tablabel = gtk_label_new (_("Firewall hits"));
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scrolled_win,
-				  tablabel);
+		tablabel);
 
 	for (i = 0; i < 4;i++)
 		 hitlogtitles[i] = _(hitlogtitles[i]);
 	hitlogclist = gtk_clist_new_with_titles (4, hitlogtitles);
 	prepare_clist (hitlogclist, NULL);
 	gtk_container_add (GTK_CONTAINER (scrolled_win), hitlogclist);
+	prepare_for_sorting (hitlogclist, 0);
 
-/* set up the clists on the dynamic rules page */
+/* Set up the clists on the dynamic rules page */
 	tablabel = gtk_label_new (_("Dynamic rules"));
 	dynamicvbox = gtk_vbox_new (FALSE, 0);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), dynamicvbox,
-				  tablabel);
+		tablabel);
 
 	denyalltitles[0] = N_(denyalltitles[0]);
 	denyallclist = gtk_clist_new_with_titles (1, denyalltitles);
 	gtk_object_set_data (GTK_OBJECT (denyallclist), "path",
 		FIRESTARTER_RULES_DIR "/firestarter/deny-all");
 	prepare_clist (denyallclist, dynamicvbox);
+/*	prepare_for_sorting (denyallclist, 1); */
 
 	allowtitles[0] = N_(allowtitles[0]);
 	allowallclist = gtk_clist_new_with_titles (1, allowtitles);
 	gtk_object_set_data (GTK_OBJECT (allowallclist), "path",
 		FIRESTARTER_RULES_DIR "/firestarter/allow-all");
 	prepare_clist (allowallclist, dynamicvbox);
+/*	prepare_for_sorting (allowallclist, 2); */
 
 	allowsmtitles[0] = N_(allowsmtitles[0]);
 	allowsmclist = gtk_clist_new_with_titles (1, allowsmtitles);
 	gtk_object_set_data (GTK_OBJECT (allowsmclist), "path",
 		FIRESTARTER_RULES_DIR "/firestarter/allow-service-machine");
 	prepare_clist (allowsmclist, dynamicvbox);
+/*	prepare_for_sorting (allowsmclist, 3); */
 
 	allowsatitles[0] = N_(allowsatitles[0]);
 	allowsaclist = gtk_clist_new_with_titles (1, allowsatitles);
 	gtk_object_set_data (GTK_OBJECT (allowsaclist), "path",
 		FIRESTARTER_RULES_DIR "/firestarter/allow-service-all");
 	prepare_clist (allowsaclist, dynamicvbox);
+/*	prepare_for_sorting (allowsaclist, 4); */
 
-	logpath = gnome_config_get_string (
-		"/firestarter/Files/logfile=/var/log/messages");
-
-	/* Check if user has read access to the log file */
-	if (fopen (logpath, "r") == NULL) {
-		gchar *string;
-		GtkWidget *dialog;
-
-		string = g_strjoin (NULL, _(
-			"Log file not found or access denied.\n"
-			"Firewall log monitoring disabled.\n\n"
-			"You can configure the path to the log file in the "
-			"\"preferences\" section."), NULL);
-		dialog = gnome_error_dialog (string);
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-		gtk_widget_set_sensitive (GTK_WIDGET (hitlogclist), FALSE);
-
-		g_free (string);
-	}
-	else {
-		gchar buf[512];
-
-		file = fopen (logpath, "r");
-		/* Move to the end of the file */
-		while (fgets (buf, 512, file) != NULL) continue;
-		gtk_timeout_add (500, poll_log_timeout, file);
-	}
+	open_logfile (gnome_config_get_string (
+		"/firestarter/Files/logfile=/var/log/messages"));
 
 	mkdir (FIRESTARTER_RULES_DIR "/firestarter", 00700);
 
@@ -769,8 +622,6 @@ main (int argc, char* argv[])
 	parse_allow_service_machine ();
 	parse_allow_service_all ();
 
-	appbar_update ("unknown");
-
 	if (!gnome_config_get_bool ("/firestarter/Program/starthidden=FALSE"))
 		gtk_widget_show_all (window);
 
@@ -783,6 +634,8 @@ main (int argc, char* argv[])
 
 	if (gnome_config_get_bool ("/firestarter/Program/startup=TRUE"))
 		restart_firewall ();
+	else
+		appbar_update ("stop");
 
 	gtk_main ();
 
@@ -790,3 +643,36 @@ main (int argc, char* argv[])
 
 	return 0;
 }
+
+/* [ save_session ]
+ * Saves the current session for later revival
+ */
+static gint
+save_session (GnomeClient       *client,
+	      gint               phase,
+	      GnomeSaveStyle     ave_style,
+	      gint               is_shutdown,
+	      GnomeInteractStyle interact_style,
+	      gint               is_fast,
+	      gpointer           client_data)
+{
+	gchar **argv;
+	guint argc;
+
+	argv = g_new0 (gchar*, 4);
+	argc = 1;
+
+	argv[0] = client_data;
+	gnome_client_set_clone_command (client, argc, argv);
+	gnome_client_set_restart_command (client, argc, argv);
+	return TRUE;
+}
+
+/* [ session_die ]
+ * Gracefully end the session
+ */
+static void
+session_die (GnomeClient *client, gpointer client_data)
+{
+	exit_firestarter ();
+}
