@@ -8,6 +8,9 @@
  *
  * The preferences notebook window
  *--------------------------------------------------------------------*/
+#include <stdio.h>
+#include <errno.h>
+
 #include "preferences.h"
 #include "firestarter.h"
 #include "service.h"
@@ -81,10 +84,15 @@ save_preferences (GnomePropertyBox *property_box, gint page_num, gint num)
 /* TODO: This function is called 2 times when apply is pressed, why? */
 
 	gchar *logpath = FIRESTARTER_RULES_DIR "/firestarter/do-not-log-ports";
-	FILE *f = fopen (logpath, "w");
+	FILE *f;
 	gchar *text;
 	gint i, j;
 
+    if ((f = fopen (logpath, "w")) == NULL) {
+       perror(logpath);
+       return;
+    }
+    
 /* General */
 	gnome_config_set_string ("/firestarter/Files/logfile",
 		gtk_entry_get_text (GTK_ENTRY (logentry)));
